#! /usr/bin/env python

# Copyright (C) 2005  Jean-Claude Rimbault & Optiman
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from pythonite.utils import *
from pythonite.templating import *
from pythonite import config
import os, sys, random, traceback

type_registry = {}
ext_registry = {}
obj_registry = {}

class Option:
    def __init__(self, req):
        try:
            self._opt = req.get_options()
	    log(2, 'options %s' % repr(self._opt))
        except AttributeError, why:
            self._opt = {}
	    log(2, 'warning: cannot read options - %s' % why)
    def __getattr__(self, attr):
        if attr.startswith('_'):
            raise AttributeError, attr
        try:
            return self._opt[attr]
        except KeyError:
            try:
                return getattr(config, attr)
            except AttributeError:
                return ''

class Context:
    def __init__(self, req):
        self.dict = {
            'request': req,
            'os': os,
            'open': open,
            'random': random,
            'sort': sort,
            'Empty': Empty,
            'session': req.session,
            'option': req.option,
	    'iteration': None,
        }
        for objname, (objclass, objarg) in obj_registry.items():
            #print objname, objclass, objarg
            if objarg:
                arg = self.eval(objarg)
                self.dict[objname] = objclass(arg, self)
            else:
                self.dict[objname] = objclass
    def get(self, key):
        if key == 'option':
            try:
                return self.find(key)
            except NameError:
                return self.dict[key]
        else:
            try:
                return self.dict[key]
            except KeyError:
                return self.find(key)
    def set(self, key, val):
        self.dict[key] = val
    def save(self, key, new):
        try:
            old = self.get(key)
        except KeyError:
            save = None
        else:
            save = (key, old)
        self.set(key, new)
        return save
    def restore(self, save):
        if save:
            key, old = save
            self.set(key, old)
    def eval(self, expr):
        log(2, 'expr='+repr(expr))
        errors = {}
        while 1:
            try:
                val = eval(expr, self.dict)
                log(2, 'return '+repr(val))
                return val
            except NameError, info:
                try:
		    name = info.args[0]
		    if "'" in name:
                        name = name.split("'")[1]
                    if errors.has_key(name):
                        return ErrorString(self, expr)
                    errors[name] = 1
                    log(2, 'find '+name)
                    self.dict[name] = self.find(name)
                except:
                    return ErrorString(self, expr)
            except:
                return ErrorString(self, expr)
    def find(self, name):
        log(2, 'find %s' % name)
        req = self.get('request')
        dir = os.path.dirname(req.filename)
        while dir != '/':
            path = os.path.join(dir, name) + config.extension
            if os.path.exists(path):
                return ext_registry[config.extension](path, self)
            path = os.path.join(dir, name) + '.txt'
            if os.path.exists(path):
                return ext_registry['.txt'](path, self)
            path = os.path.join(dir, name) + '.csv'
            if os.path.exists(path):
                return ext_registry['.csv'](path, self)
            path = os.path.join(dir, name) + '.py'
            if os.path.exists(path):
                return ext_registry['.py'](path, self)
            dir = os.path.dirname(dir)
        raise NameError, name

class ErrorString:
    def __init__(self, ctx, expr):
        etype, value, tb = sys.exc_info()
        #traceback.print_exc(file=open('/tmp/t','a'))
        #self.str = '*** "%s" %s: %s ***' % (expr, exc_info[0].__name__, exc_info[1])
	lines = traceback.format_exception(etype, value, tb, limit=1)
	msg = ' '.join(lines[1:]).replace('\n', '')
        errors = ctx.get('request').errors
        self.num = len(errors)+1
	self.msg = '?%d? "%s" %s' % (self.num, expr, msg)
        errors.append(self.msg)
        log(0, 'error '+self.msg)
    def __str__(self):
        return '?%d?' % self.num
    def __getitem__(self, i):
	raise IndexError, i
    def __iter__(self):
        return iter([])
    def __len__(self):
        return 0        # => is false in condition tests

def makefile(path, context):
    if os.path.isdir(path):
        constructor = ext_registry['/']
    else:
        suffix = os.path.splitext(path)[-1]
        try:
            constructor = ext_registry[suffix.lower()]
        except KeyError:
            constructor = ext_registry['']
    return constructor(path, context)

def register_ext(ext, objclass):
    ext_registry[ext] = objclass
    type_registry.setdefault(objclass.objtype, []).append(ext)

def register_obj(objname, objclass, objarg):
    obj_registry[objname] = (objclass, objarg)
