#! /usr/bin/env python

# Copyright (C) 2005  Jean-Claude Rimbault & Optiman
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from SimpleHTTPServer import SimpleHTTPRequestHandler
from BaseHTTPServer import HTTPServer
from pythonite.core import handle_request, __version__
from pythonite.utils import Empty
from pythonite import config
import os, time, urllib

class Request:
    uri = 'http://localhost/'
    path = '/'
    args = ''
    rootdir = '/var/www/html/'
    filename = '/var/www/html/index.htm'
    status = 200
    request_time = 0
    def get_options(self):
        return {}

class PythoniteHandler(SimpleHTTPRequestHandler):
    def do_POST(self):
        ctype = self.headers["content-type"]
        if ctype != 'application/x-www-form-urlencoded':
            raise ValueError, "Don't know how to handle content-type %s" % ctype
        clen = int(self.headers["content-length"])
        data = self.rfile.read(clen)
        self.do_GET('POST', data)
    def do_GET(self, method='GET', data=''):
        req = Request()
        req.method = method
        try:
            req.path, req.args = self.path.split('?', 1)
        except ValueError:
            req.path, req.args = self.path, ''
        req.rootdir = os.getcwd()
        req.filename = os.path.join(req.rootdir, req.path[1:])
        if os.path.isdir(req.filename) and not req.path.endswith('/'):
            host = self.headers.get('Host', 'localhost:8000')
            uri = 'http://' + host + self.path + '/'
            self.send_response(302)
            self.send_header("Location", uri)
            self.end_headers()
            return
        if req.path.endswith('/'):
            req.path += config.index
        if req.path.endswith(config.extension):
            req.filename = os.path.join(req.rootdir, req.path[1:])
            req.headers_in = self.headers
            req.headers_out = {}
            req.request_time = time.time()
            host = self.headers.get('Host', 'localhost:8000')
            req.uri = 'http://' + host + self.path
            req.fields = []
            if data:
                data = req.args + '&' + data
            else:
                data = req.args
            for arg in data.split('&'):
                field = Empty()
                try:
                    field.name, field.value = arg.split('=', 1)
                    field.value = urllib.unquote_plus(field.value)
                except ValueError:
                    field.name, field.value = arg, ''
                req.fields.append(field)
            data = handle_request(req)
            ctype = 'text/html'
            self.send_response(req.status)
            self.send_header("Content-type", ctype)
            self.send_header("Content-Length", len(data))
            for key, val in req.headers_out.items():
                self.send_header(key, val)
            self.end_headers()
            self.wfile.write(data)
        else:
            SimpleHTTPRequestHandler.do_GET(self)

def run(addr='', port=8000):
    server_address = (addr, port)
    PythoniteHandler.protocol_version = "HTTP/1.0"
    httpd = HTTPServer(server_address, PythoniteHandler)
    sa = httpd.socket.getsockname()
    print "Pythonite", __version__, "serving HTTP on", sa[0], "port", sa[1], "..."
    httpd.serve_forever()

if __name__ == '__main__':
    import sys

    if sys.argv[1:]:
        port = int(sys.argv[1])
    else:
        port = 8000
    os.chdir('testsite')
    run('', port)
