#! /usr/bin/env python

# Copyright (C) 2005  Jean-Claude Rimbault & Optiman
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from pythonite.utils import *
import Cookie, os, time

class Session(Empty):
    _cookie_name = 'PYSESSIONID'
    _session_dir = '/tmp/.session'
    def __init__(self, req):
        try:
            sesstime = int(req.option.sesstime)
        except ValueError:
            sesstime = 300
        try: # old version
            req = req._req
        except AttributeError:
            pass
        now = time.time()
        try:
            cookie = req.headers_in['cookie']
            self._id = cookie.split(';')[0].split('=', 1)[-1]
        except KeyError:
            self._id = '%.6f' % now
        cookie = '%s=%s; path=/; expires=%s' % (self._cookie_name, self._id, Cookie._getdate(sesstime))
        #req.headers_out.add('Set-cookie', cookie)
        req.headers_out['Set-cookie'] = cookie
        self._session_path = os.path.join(self._session_dir, self._id)
        if not os.path.exists(self._session_dir):
            os.mkdir(self._session_dir)
        try:
            file = open(self._session_path, 'r')
        except IOError:
            pass
        else:
            for line in file.readlines():
                key, val = line[:-1].split('=', 1)
                setattr(self, key, eval(val))
            file.close()
        # old sessions cleanup (files older than MARK)
        mark_path = os.path.join(self._session_dir, 'MARK')
        if os.path.exists(mark_path):
            mark_time = os.path.getmtime(mark_path)
        else:
            mark_time = now - sesstime
        if now - mark_time >= sesstime:
            for name in os.listdir(self._session_dir):
                path = os.path.join(self._session_dir, name)
                if os.path.getmtime(path) <= mark_time:
                    log(3, 'removing old session %s' % path)
                    os.remove(path)
            open(mark_path, 'w').write('')
    def save(self):
        file = open(self._session_path, 'w')
        for key, val in self.__dict__.items():
            if not key.startswith('_'):
                file.write('%s=%s\n' % (key, repr(val)))
        file.close()
